! Info: tests all facilities in omsourceutils_mod
program main
   use omsourceutils_mod
   implicit none

   call testSorted
   call testMedian
   call testFractionBoxInCircle

contains

   subroutine testSorted
      real(single),  parameter :: uv(5)=(/5.,1.,3.,2.,4./)
      real(single),  parameter :: sv(5)=(/1.,2.,3.,4.,5./)
      integer(int32),parameter :: si(5)=(/2,4,3,5,1/)

      call assertOn(any(sortedIndexOf(uv) /= si),"failing sortedIndexOf()")
      call assertOn(any(sorted(uv) /= sv),"failing sorted()")
   end subroutine testSorted



   subroutine testMedian
      real(single),  parameter :: uv1(5)=(/5.,1.,3.,2.,4./)
      real(single),  parameter :: uv2(6)=(/3.,2.,1.,1.,2.,3./)

      ! if OK, then medianOfSorted() is also tested, because of 
      ! testSorted()
      call assertOn(medianOfUnsorted(uv1) /= 3.,"failed median of v(5)")
      call assertOn(medianOfUnsorted(uv2) /= 2.,"failed median of v(5)")
   end subroutine testMedian



   ! Info: test the fractionBoxInCircle by comparing
   !       how a the numerical approach differs from analytic expression
   subroutine testFractionBoxInCircle()
      real(single) :: ce(2)
      real(single) :: p1(2)
      real(single) :: p2(2)
      real(single) :: R
      real(single) :: f
      real(single) :: fPix
      real(single) :: fCircle
      integer      :: i,j

      ce=5.
      R =5.
      f =0.
      fCircle=PI*R**2

      ! loop over pixels of unit-size, and sum the fractions
      do i=1,10
         do j=1,10
            p1=(/i,j/)-1. ! lower-left corner of pixel
            p2=(/i,j/)    ! upper-right corner of pixel
            fPix=fractionBoxInCircle(ce,R,p1,p2)
            f=f+fPix
         end do
      end do

      print '("fractionBoxInCircle against analytic:",2F20.8)', fCircle,f
      call assertOn(abs(fCircle-f) > 1.e-3,"Bad fractionBoxInCircle")
   end subroutine testFractionBoxInCircle



   ! lo,hi are in untis of Radius of circle
   subroutine handleBox(lo,hi, txt)
      real(single), intent(in) :: lo(2)
      real(single), intent(in) :: hi(2)
      character(*), intent(in),optional :: txt

      real(single) :: Ainter, fInter
      real(single) :: Abox 

      ! get the fraction of area of circle that intersects with box
      Finter   = fractionBoxInUnitCircle(lo,hi)
      Ainter   = Finter/PI
      Abox     = (hi(1)-lo(1))*(hi(2)-lo(2))

      if (present(txt)) then
         print '(3F15.7,2A)', Finter,Ainter,Abox,' ',txt
      else
         print '(3F15.7)',    Finter,Ainter,Abox
      end if
   end subroutine handleBox

end program main
