// -*-C++-*-
//
// File:      OmSourceBrowser.cc
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Fri Sep  6 11:14:45 MEST 2002
// Usage:     -
// Info:      -
#include "OmSourceParameters.h"
#include <qspinbox.h> 
#include "OmSourceBrowser.h"
#include "OmSourceEditor.h"
//#include "OmSourceParameters.h"
#include "OmSourceRegions.h"
#include "OmImageDisplay.h"
#include "OmProgressDialog.h"
#include "OmSourceLog.h"
#include <param.h>
#include <errstr.h>
#include <qlistbox.h>
#include <qcombobox.h>
#include <qgroupbox.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qstatusbar.h>
#include <qprogressbar.h>
#include <qstring.h>
#include <qtooltip.h>
#include <math.h>
#include <qinputdialog.h>
#include<assert.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qprinter.h> 
#include <qpainter.h>
#include <qfont.h> 
#include <qpaintdevicemetrics.h> 
#include <qapplication.h>

#include <omdetect.h>
#include <ommag.h>


// --- Glue prototypes ---
extern F90MODVARDEF(OmSourceParameters, omsource_glue, mysources);
//extern F90MODVARDEF(ObsDetails, omsource_glue, myobsdetails);
extern F90MODVARDEF(PhotometryParameters, omsource_glue, myphotometryparameters);

extern F90MODVARDEF(float, omsource_glue, mycalradius);


F90MODPROCPROTO(void, omsource_glue, init)(const char[256]);
F90MODPROCPROTO(void, omsource_glue, setimagefilename)(const char[256]);
F90MODPROCPROTO(void, omsource_glue, setnewsourcelistfilename)(const char[256]);
F90MODPROCPROTO(void, omsource_glue, setoldsourcelistfilename)(const char[256]);

F90MODPROCPROTO(void, omsource_glue, finish)(const char[] );



F90MODPROCPROTO(void,omsource_glue, loadsources)(const char []);

F90MODPROCPROTO(void,omsource_glue, recompute)(const int& id);
F90MODPROCPROTO(void,omsource_glue, remove)(const int& id);
F90MODPROCPROTO(void,omsource_glue, addregions)(OmSourceRegions*);

void ComputeAverageMagnitude(float* magnitude, float* error, int size, float& mean, float& standardDeviation);
void GetOldFileName(QWidget *parent, char imageFileName[], char *parameter, char *title, char *filter);
void GetNewFileName(QWidget *parent, char imageFileName[], char *parameter, char *caption, char *title);



/*******************************************************
* constructor
* This is the main widget of the application
*******************************************************/
OmSourceBrowser::OmSourceBrowser(QWidget* parent, const char* name) : 
    QWidget(parent, name), _current(0)
{ 


    OmProgressDialog* reporter = 0;
    reporter = new OmProgressDialog("Initialising omsource - please wait", 6);
 
    loadFileNames();
    _changed = FALSE;
    _count = 0;
    _magnitude = new float[100];
    _magnitudeError = new float[100];
    resize( 597, 800 ); 
      
   //    QProgressDialog *progress = reporter->GetProgressDialog();
    //    QObject::connect( progress, SIGNAL(cancelled()), this, SLOT(Cancel()));
     
  
   
    reporter->setProgress(1);
    
  
 


  // create the image buttons
    {
    QGroupBox* group = new QGroupBox( this );
    //    group->setGeometry( QRect( 10, 241, 121, 100 ) );
    group->setGeometry( QRect( 10, 220, 121, 100 ) );
    



    group->setTitle("Imager");

    _viewAllBtn=new QPushButton(group);
    _viewAllBtn->setGeometry( QRect( 10, 20, 100, 35 ) );
    _viewAllBtn->setText("View all");
    QToolTip::add( _viewAllBtn , QRect( 0, 0,   _viewAllBtn->width() / 2, 
	 _viewAllBtn->height() ), "View the whole image");


    _addSourcesBtn = new QPushButton(group);
    _addSourcesBtn->setGeometry( QRect( 10, 60, 100, 35 ) );
    _addSourcesBtn->setText("Add Sources");

    QToolTip::add( _addSourcesBtn , QRect( 0, 0,   _addSourcesBtn->width() / 2, 
	 _addSourcesBtn->height() ), "Select additional sources on image to add to source list");


    }
    connect(_viewAllBtn, SIGNAL(clicked()), this, SLOT  (viewAll()));
    connect(_addSourcesBtn, SIGNAL(clicked()), this, SLOT  (addSources()));



    /*****************************************************************
    * Get the input image file name
    ****************************************************************/
    //    char fileName[256] =  {' '};

    //    GetOldFileName(0, fileName, "imageset", "Please select an image file", "IMAGES_ (*.FIT)");
    //    strcpy(_imageFileName, fileName);
   

   
    //    strcpy(fileName, " ");
    //    GetOldFileName(0, fileName, "oldsrclistset", "Please select an old source-list file", "(*.FIT)");
    //    strcpy(_oldSourceListFile, fileName);
    
   
    //        F90MODPROC(omsource_glue, setoldsourcelistfilename)(_oldSourceListFile);
  
    reporter->setProgress(2);
 
    F90MODPROC(omsource_glue, init)(_imageFileName);
  
     reporter->setProgress(3);
  


    // create the sourceeditor
 
    _editor = new OmSourceEditor(this);
  

    _editor->setGeometry( QRect( 140, 20, 480, 420 ) );     
  //_editor->setEnabled(false);
    connect(_editor,SIGNAL(changed(const OmSourceParameter&)),
	  this,   SLOT  (recompute(const OmSourceParameter&)));
    connect(_editor,SIGNAL(requestedDelete()),
	  this,   SLOT  (deleteSource()));

    reporter->setProgress(4);
 

  {
    QGroupBox* group = new QGroupBox( this );
    //    group->setGeometry( QRect( 10, 241, 121, 100 ) );
    group->setGeometry( QRect( 10, 460, 620, 180 ) );
    


    // create the logger
    _logger = new OmSourceLog(group);
 
    _logger->setGeometry( QRect(  10, 10, 600, 100 ) );





    _averageMagList  = new QComboBox( group);


    _averageMagList->setGeometry(10, 130, 350, 30);
 
 
    _averageMagList->setDisabled(TRUE);
    

    /*********************************************************
    * A button to print its contents
    ********************************************************/
    _printAveragesBtn = new QPushButton("Print averages", group);
    _printAveragesBtn->setGeometry(380, 130, 100 , 35);
    QObject::connect(_printAveragesBtn, SIGNAL(clicked()), this, SLOT (print1()) );

    QToolTip::add( _printAveragesBtn , QRect( 0, 0,   _printAveragesBtn->width() / 2, 
	 _printAveragesBtn->height() ), "Print the computed average magnitudes stored in the combo box");

    _printAveragesBtn->setDisabled(TRUE);

 


   /*********************************************************
   * A button to print the logger
   ********************************************************/
    _printLogBtn = new QPushButton("Print table", group);
    _printLogBtn->setGeometry(380 + _printAveragesBtn->width() + 10, 
            130, 100 , 35);
    QObject::connect(_printLogBtn, SIGNAL(clicked()), this, SLOT (print2()) );

    QToolTip::add( _printLogBtn , QRect( 0, 0,   _printLogBtn->width() / 2, 
	 _printLogBtn->height() ), "Print the photometry calculations stored in the table");

    _printLogBtn->setDisabled(TRUE);


  }

    // Quit button
    QPushButton* myQuitButton = new QPushButton("Quit", this);
    
    myQuitButton->setGeometry(490, 670, 100 , 35);


    QObject::connect(myQuitButton, SIGNAL(clicked()), parent, SLOT (Exit()) );
	    //        QObject::connect(myQuitButton, SIGNAL(clicked()), parent, SLOT ( qApp->quit() ) );

    






    //    ObsDetails& obsDetails = F90MODVAR(omsource_glue, myobsdetails);
    //   ImageDetails& imageDetails = F90MODVAR(omsource_glue, myimagedetails);
  
 

    /************************************************ 
    * create the list
    ************************************************/

    _list = new QComboBox( this, "OmSourceList" );

    _list->setGeometry( QRect( 10, 0,  this->width() - 10,  20) );
  

    {
    QGroupBox* group = new QGroupBox( this );
    //    group->setGeometry( QRect( 10, 241, 121, 100 ) );
    group->setGeometry( QRect( 10, 340, 121, 100 ) );
    


    group->setTitle("Source selection");



    QLabel* label = new QLabel( group, "label" );
    label->setGeometry( QRect( 10, 20, 60, 35 ) ); 
    label->setText( tr( "Source no " ) );
    QToolTip::add( label, QRect( 0, 0, label->width() / 2, 
		       label->height() ), "Source number to edit");


    /******************************************************************
    * A spin box to set the source number
    *****************************************************************/

    _spinBoxNumber = new QSpinBox(group, "a");


    _spinBoxNumber->setGeometry(10, 30, 60, 25);
 
 

    _spinBoxNumber->setMinValue(0);
    _spinBoxNumber->setMaxValue(_list->count());
    _spinBoxNumber->setValue(0);
    reporter->setProgress(5);
 

    //    connect(_spinBoxNumber, SIGNAL(valueChanged(int)), this, SLOT(  edit1(int)));
    connect(_spinBoxNumber, SIGNAL(valueChanged(int)), this, SLOT(  setListBoxItem(int)));
    connect(_list, SIGNAL(highlighted(int)), this, SLOT(  SetSpinBoxValue(int)));


    QToolTip::add( _spinBoxNumber, QRect( 0, 0, _spinBoxNumber->width() / 2, 
		       _spinBoxNumber->height() ), "Change the source number to edit");

    _spinBoxNumber->show();




    _editSource = new QPushButton(group);
    _editSource ->setGeometry(10, 60, 100, 35);
    _editSource->setText("Select source");
    QToolTip::add( _editSource , QRect( 0, 0,   _editSource->width() / 2, 
     _editSource->height() ), "Select the source for redoing aperture photometry");
    connect(_editSource,SIGNAL(clicked()),
	  this,   SLOT  (editSource()));
        
 
    }


   
    reporter->setProgress(6);  
    delete reporter;


}
/*******************************************************
* 
*******************************************************/
void OmSourceBrowser::Initialise()
{ 

  
  
 
    // load display
    _display = new OmImageDisplay;


   
    if (!_display->isUp())
        errstr << "Could not launch Imageviewer(ds9)" << Sas::error("NoDisplayer");

 
   
     
    _display->displayImage(string(_imageFileName));
 
    
               

    OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);
  
    if (mySources.length() == 0)
    {
    
        /*************************************************************
        * Can either run omdetect or select sources manually
        ************************************************************/


        QMessageBox mb( "omsource",
                    "There are no sources to display\n"
                    "Do you wish to run omdetect or select sources manually?",
                    QMessageBox::Information,
                    QMessageBox::Yes | QMessageBox::Default,
                    QMessageBox::No,
                    QMessageBox::Cancel | QMessageBox::Escape );
        mb.setButtonText( QMessageBox::Yes, "Run omdetect" );
        mb.setButtonText( QMessageBox::No, "Select manually" );
        mb.setButtonText( QMessageBox::Cancel, "Quit program" );

        switch (mb.exec())
	{
	    case (QMessageBox::Yes):
	    
	        /*************************************************
	        * Run omdetect
	        *************************************************/    
	        emit SetStatusBarMessage("Running omdetect- pleased wait");
               
	              
                sourceDetection(_imageFileName); 
	       
             
	        emit SetStatusBarMessage("omsource version 1.1- sources can now be edited");
	       
	     		        
	       
	        break;
	    case (QMessageBox::No):
            
	        
                
	        addSources();
                
                break;
	    case (QMessageBox::Cancel):
	        qApp->quit();
	       
	        break;
	    default:
	        break;           
        }

    } 




 
    /*********************************************************
    * Load the list box with the parameters for each source
    ********************************************************/
    refreshListBox();
    //  connect(_list,SIGNAL(highlighted(int)),this,SLOT(edit(int)));

  

    connect(_list, SIGNAL(activated(int)), this, SLOT(edit(int)));


    QString string;
 
  
    string.sprintf("Source Browser (No of sources=%d)", _list->count());

    SetCaption(string);

   

}

/******************************************************
* destructor
*******************************************************/

OmSourceBrowser::~OmSourceBrowser()
{
    _display->down();
    delete _display;

    delete [] _magnitude;
    delete [] _magnitudeError;

    /************************************************
     * Need to get the new source-list file name
     ***********************************************/

    //    char newSourceListFile[128] = {' '};    // For the new source list  file
    //    string newSourceListFile = stringParameter("newsrclistfile");
    /*****************************************************************
    * Get the input image file name
    ****************************************************************/
    //    GetNewFileName(this, newSourceListFile, "newsrclistset", "omsource", "Please enter the name for the new source-list file"); 
       
   
    F90MODPROC(omsource_glue, finish)(_newSourceListFile);

    //    F90MODPROC(omsource_glue, finish)();


}
/******************************************************
* loadFileNames
* Purpose:- Gets the name of the input and output file names
* given as parameters
*******************************************************/

void OmSourceBrowser::loadFileNames()
{
    string fileName = stringParameter("imageset");

    strcpy(_imageFileName, fileName.c_str());
    fileName = stringParameter("oldsrclistset");

    strcpy(_oldSourceListFile, fileName.c_str());
    

    fileName = stringParameter("newsrclistset");

    strcpy(_oldSourceListFile, fileName.c_str());
    


}
/******************************************************
* edit
*******************************************************/

void OmSourceBrowser::edit(int id)
{
  
  
    OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);
 
 
    id += 1; // sources start at 1, but listbox at 0
    if (mySources.length() < id) return;

    _sourceNo = id;
  

    if (_current != id)
    {

        _logger->restart();
        _count = 0;
    //    _magnitudeLabel->hide();
        _averageMagList->setDisabled(true);
        _printLogBtn->setDisabled(true);
        _printAveragesBtn->setDisabled(true);

         
  
    }  
    _current = id;
 
       
    _editor->setEnabled(false);
    

    _editor->edit(id, mySources[_current]);
   

    _display->zoomTo(mySources[_current]);
    _editor->setEnabled(true);
    _logger->add(mySources[_current]);


    /**************************************************
    * Recompute the average magnitude 
    **************************************************/

    
   _magnitude[_count] = mySources[_current].magnitude;
   _magnitudeError[_count] = mySources[_current].magnitudeerror;
   
 
    if (_count > 0) 
    {

        float mean, standardDeviation;
       
        ComputeAverageMagnitude(_magnitude, _magnitudeError, _count + 1, mean, standardDeviation);
        QString text;
        text.sprintf("Average magnitude for source %d=%5.2f+/-%5.2f (n=%d)", 
               id, mean, standardDeviation, _count + 1); 
        
        
	//        _averageMagList->show();
        _averageMagList->setEnabled(true);

	//        _printLogBtn->show();
        _printLogBtn->setEnabled(true);
        _printAveragesBtn->setEnabled(true);


        _averageMagList->insertItem(text);
        _averageMagList->setEditText(text);

      //      _magnitudeLabel->setText(text);
      //      _magnitudeLabel->show();
        _averageMagList->setCurrentItem(_averageMagList->count() - 1); 
    }
    _count++;
   
   
}
/**********************************
* ComputeAverage
* Calculates the weighted average and
* standard deviation 
**********************************/
void ComputeAverageMagnitude(float* magnitude, float* error, 
              int size, float& mean, float& standardDeviation)
{    
       
    bool useWeights;
    int i;
      
    if (size == 1) 
    {
        mean = magnitude[0];
	standardDeviation = error[0];
	return;
    }


    double* flux = new double[size];
    double* fluxError = new double[size];

    int nData = -1;
    for (i = 0; i <  size; i++)
    {
        if (magnitude[i] >= 0.0 && magnitude[i] < 100.0)
        {
	    nData = nData + 1; 
            flux[nData] = pow(10.0, (-0.4 * magnitude[i]));                 
            fluxError[nData] = flux[nData] * error[i];
	}
    }   
        
    if (++nData < 2) 
    {
        mean = magnitude[0];
        standardDeviation = error[0];
        delete [] flux;
        delete [] fluxError;
        return;
    }


   
    useWeights = TRUE;
   

   
    double sum1 = 0.0;
    double sum2 = 0.0;
    double sum3 = 0.0;
 

    for( i = 0; i < nData; i++)
    {
       
        if (fluxError[i] <= 0.0)
	{
	    useWeights = FALSE;
            sum1 = double(size);
            sum3 = double(size);
            break;
        }
    } 

    
    for (i = 0; i <  nData; i++)
    { 
       
        if (useWeights)
        {
	    double weight = 1.0 / (fluxError[i] * fluxError[i]);
	    sum1 += weight;
	    sum2 += flux[i] * weight;
	    sum3 += weight * weight;
        }
        else
        {
	    sum2 += flux[i];
          
        }

    }

    mean = sum2 / sum1;  // The mean

    sum2 = 0.0;

    for (i = 0; i <  nData; i++)
    {
       
   
        double difference = flux[i] - mean;       
        if (useWeights) 
	{
	    double weight = 1.0 / (fluxError[i] * fluxError[i]);
            sum2 += weight * difference * difference;
        }
        else
	    sum2 += difference * difference;
       
    }

    double d;

    if (useWeights) 
        d = sum1 - sum3 / sum1;
    else
        d = float(size - 1); 
    
    standardDeviation = sqrt(sum2 / d);  //  The standard deviation
    delete [] flux;
    delete [] fluxError;
    
    standardDeviation = standardDeviation / mean;  
    mean = -2.5 * log10(mean);
}
/******************************************************
* edit1
*******************************************************/
void OmSourceBrowser::edit1(int num) 
{
    edit(num - 1);
};
/******************************************************
* recompute
*******************************************************/


void OmSourceBrowser::recompute(const OmSourceParameter& s)
{

    OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);
  
    mySources[_current] = s;
    F90MODPROC(omsource_glue,recompute)(_current);
    _display->toRegion(mySources);

    _changed = TRUE;
    edit(_current - 1);

}
/******************************************************
* deleteSource
*******************************************************/
void OmSourceBrowser::deleteSource()
{
  // update the scrollview AND mySources!
    OmSourceParameters& mySources=F90MODVAR(omsource_glue,mysources);
    if (!_current || mySources.length() < _current) return;

  // 
    F90MODVAR(omsource_glue, remove)(_current);
    
   

    QString string;
    string.sprintf("Source browser (No of sources=%d)", _list->count());
    _changed = TRUE; 
    emit SetCaption(string);

}
/***********************************************************
* viewAll
************************************************************/
void OmSourceBrowser::viewAll()
{
    OmSourceParameters& mySources=F90MODVAR(omsource_glue,mysources);
    _display->send("zoom to fit");
    _display->toRegion(mySources);
}
/********************************************************
 * addSources
 *******************************************************/
void OmSourceBrowser::addSources()
{

    HideParent();
    OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);

    _display->toRegion(mySources);
    _display->send("regions color green");
    _display->send("raise");

     
    viewAll();

    
    QMessageBox mb( "omsource",
                "Please select additional sources in the image by centering\n"
                " the cursor over the source and clicking the mouse button.\n\n"
		"When finished press the 'Finished' button.\n\n"
		"The center coordinates of these regions will then\n"
			"be used to compute additional sources in your sourcelist.",
                  

                    QMessageBox::Information,
			QMessageBox::Yes | QMessageBox::Default,

		    //                    QString::null,  QString::null);              
                    QMessageBox::No,
                    QMessageBox::Cancel | QMessageBox::Escape );
 
    
    mb.setButtonText( QMessageBox::Yes, "Finished" );
      

    //    this->parent->hide(); // NB   
   
    mb.move(0, 0);
    mb.exec();
    //    this->parent->show();  // NB
   
         

  GeoRegion gReg = _display->regionSelection(ImageDisplayer::IMAGE, "yellow");

 
   



  // count
    int nPoints=0;
    {
    for(vector<Component>::const_iterator it = gReg.begin();
	it != gReg.end(); it++)
      nPoints += (*it).size();
    }
    if (!nPoints)
    {

        ShowParent();
        emit SetStatusBarMessage("omsource version 1.1 (No new sources were selected)");
      
        return;
    }
    else 
        emit SetStatusBarMessage("omsource version 1.1");
  


  // now we contruct a dopevector...

    OmSourceRegions points(1, nPoints);
    {
    unsigned long idx = 1;
    for(vector<Component>::const_iterator it = gReg.begin();
	it != gReg.end(); it++) { // should be one only....
        const Component& component=(*it);
        for(vector<FShape*>::const_iterator ic = component.begin();
	    ic != component.end(); ic++) points[idx++] = (*ic)->refPoint();
    }
  }
  // ... pass it to fortran to compute their properties ...
    

    F90MODPROC(omsource_glue, addregions)(&points);
 


  // refresh
    refreshListBox();
 

  

    QString string;
 

  
    string.sprintf("Source browser (No of sources=%d)", _list->count());

    emit SetCaption(string);



    ShowParent();
}
/******************************************************************
* refreshListBox
* - loads the list box that displays the parameters for each source
*****************************************************************/
void OmSourceBrowser::refreshListBox()
{

  
    /***************************************************************
    * Load the sources
    ***************************************************************/
    OmSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);
    _display->toRegion(mySources);

  //  _list->setFocus();
    _list->clear();
    _list->blockSignals(true);


  //  const QString prefix("Source ");


    for (int i = mySources.lbound(); i<= mySources.ubound(); i++) 
    {
        QString s;
 
        s.sprintf("Source %5d (%6.1f,%6.1f), count-rate=%10.2f+/%5.2f,  instrumental magnitude=%5.2f+/-%5.2f", 
              i, 
              mySources[i].xcoord, mySources[i].ycoord, 
		  mySources[i].countrate,  mySources[i].countrateerror, 
              mySources[i].magnitude, 
              mySources[i].magnitudeerror);  
            

        _list->insertItem(s);
   
    }
    _spinBoxNumber->setMinValue(1);
 
    _spinBoxNumber->setMaxValue(_list->count());
    _spinBoxNumber->setValue(1);
 
    _list->blockSignals(false);


    if (_current>mySources.length()) _current = mySources.length();
    if (_current) {
        _list->setCurrentItem(_current - 1);
        edit(_current-1);
    }
  

}
/******************************************************
* print
******************************************************/
void OmSourceBrowser::print()
{
#ifndef QT_NO_PRINTER

    QPrinter printer; 
    if ( printer.setup(this) ) {                // opens printer dialog
        printer.setFullPage(TRUE);              // we'll set our own margins
	QPainter p;
	p.begin( &printer );                    // paint on printer
	p.setFont( p.font() );
	QFontMetrics fm = p.fontMetrics();
	QPaintDeviceMetrics metrics( &printer ); // need width/height
                                                 // of printer surface
    
        /*********************************************
	* Print what ever
	********************************************/
        (this->*Print)(&printer, &p,  &fm,  &metrics);

      

	p.end();                                // send job to printer
    }
#endif
}
/*********************************************************
* print1
**********************************************************/
void OmSourceBrowser::print1()
{
    Print = &OmSourceBrowser::printAverages;
    print();
}
/*********************************************************
* print2
**********************************************************/
void OmSourceBrowser::print2()
{

      
    Print = &OmSourceBrowser::printLog;
    print();
}

/*********************************************************
* printAverages
* Prints the contents of the combo box that stores
* the average magnitudes
*********************************************************/
void OmSourceBrowser::printAverages(QPrinter* printer, QPainter* p,  
               QFontMetrics* fm,  QPaintDeviceMetrics* metrics)
{
    const int MARGIN = metrics->logicalDpiX() / 2; // half-inch margin
    int yPos        = MARGIN;               // y position for each line


    QString text;
    text.sprintf("Source no %d", _sourceNo);
    p->drawText (MARGIN, yPos, text); 
    yPos += 2 * fm->lineSpacing();

    for( int i = 0 ; i < _averageMagList->count(); i++ ) 
    {
        if ( printer->aborted() )
            break;
        if ( yPos + fm->lineSpacing() > metrics->height() - MARGIN ) 
        {
                // no more room on this page
                if ( !printer->newPage() )          // start new page
                    break;                          // some error
                yPos = MARGIN;                      // back to top of page
        }
        p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, _averageMagList->text(i));
        yPos += fm->lineSpacing();
    }

}
/*********************************************************
* printLog
* Prints the contents of the table that stores the
* user photometry calculations
*********************************************************/
void OmSourceBrowser::printLog(QPrinter* printer, QPainter* p,  
               QFontMetrics* fm,  QPaintDeviceMetrics* metrics)
{
    const int MARGIN = metrics->logicalDpiX() / 2; // half-inch margin
    int yPos        = MARGIN;               // y position for each line


    QString text;
    text.sprintf("Source no %d - photometry calculations", _sourceNo);
    p->drawText (MARGIN, yPos, text); 
    yPos += 2 * fm->lineSpacing();
    p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, _logger->getLine(0));
    yPos += fm->lineSpacing();


    for( int i = 1 ; i < _logger->numRows(); i++ ) 
    {
        if ( printer->aborted() )
            break;
        if ( yPos + fm->lineSpacing() > metrics->height() - MARGIN ) 
        {
                // no more room on this page
                if ( !printer->newPage() )          // start new page
                    break;                          // some error
                yPos = MARGIN;                      // back to top of page
        }
        p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, _logger->getLine(i));
        yPos += fm->lineSpacing();
    }

}
/************************************************************
* sourceDetection
* purpose:  Runs the source detection program, omdetect,
*           on the input image tro 
*************************************************************/
void OmSourceBrowser::sourceDetection(char imageFile[]) 
{
    char tempFile[L_tmpnam];

    tmpnam(tempFile);               // A temporary file to store the output from omdetect 
    
   
   

    Info::Task *t;
    t = task("omdetect");
    t->setParameter("set", imageFile);
   

    t->setParameter("outset", tempFile);
  

    t->exec(true);

    
    t = task("ommag");
    t->setParameter("set", tempFile);
    t->exec(true);
 
    // Load the sources
    F90MODPROC(omsource_glue, loadsources)(tempFile);
    t = task("omsource");
}
/************************************************************
* GetOldFileName
*************************************************************/
void GetOldFileName(QWidget *parent, char oldFileName[], char *parameter, char *title, char *filter)
{ 
    string fileName = stringParameter(parameter);
    

    if (fileName == string(""))
    {

        /**************************************************
	* Display get file name dialog
	*************************************************/ 

        bool okay = FALSE;
        QString text;
   

        while (!okay)
	{        
            text = QFileDialog::getOpenFileName( QString::null, filter, 
                       parent , " ", title);
  
            
            if ( text.isEmpty() )
            {  

	
                int ret = QMessageBox::critical(0, "No file has been selected",
					      "Do you wish to try again (yes), or exit (No/Cancel)",
					      QMessageBox::Yes | QMessageBox::Default,
                                              QMessageBox::No,
                                              QMessageBox::Cancel | QMessageBox::Escape );

              
              switch (ret)
	      {
	      case   (QMessageBox::Yes):
		      okay = FALSE; 
		      break;
	      case   (QMessageBox::No):
		      okay = TRUE; 
		      break;
	      case   (QMessageBox::Cancel):
		      okay = TRUE; 
		      break;
	      default:
		break;
	      }
            }
            else
	 break;
        
    
             
        }
              

        

        fileName = text;
     
    }
        
    strcpy(oldFileName, fileName.c_str());
    

 
}
/************************************************************
* GetNewFileName
*************************************************************/
 
void GetNewFileName(QWidget *parent, char newFileName[], char *parameter, char *title, char *caption)
{ 
    string fileName = stringParameter(parameter);

   
   

    if (fileName  == string(""))
    {
        bool okay = FALSE;

        while (!okay)
	{
            fileName = QInputDialog::getText( caption,  title, QString::null, &okay, parent );
            
            if (QFile::exists(fileName.c_str()))
	    {
	      int ret = QMessageBox::critical(0, "File already exists",
					      "Do you wish to type in another file (yes), or exit (No/Cancel)",
					      QMessageBox::Yes | QMessageBox::Default,
                                              QMessageBox::No,
                                              QMessageBox::Cancel | QMessageBox::Escape );
              switch (ret)
	      {
	      case   (QMessageBox::Yes):
		      okay = FALSE; 
		      break;
	      case   (QMessageBox::No):
		      okay = TRUE; 
		      break;
	      case   (QMessageBox::Cancel):
		      okay = TRUE; 
		      break;
	      default:
		break;
	      }
	 }

	}
          
            
     
    }
   
    
    strcpy(newFileName, fileName.c_str());
    
 
}
















