// -*-C++-*-
//
// File:      OmSourceEditor.cc
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Thu Sep  5 14:10:13 MEST 2002
// Usage:     -
// Info:      -

#include "OmSourceParameters.h"
#include "OmSourceEditor.h"
#include "AngleFormatter.h"
#include <qlabel.h>
#include <qlineedit.h>
#include <qlayout.h>
#include <qgroupbox.h>
#include <qpushbutton.h>
#include <qspinbox.h>

#include <qtooltip.h> 
#include <stdio.h>

extern F90MODVARDEF(PhotometryParameters, omsource_glue, myphotometryparameters);

/****************************************************
*
****************************************************/

OmSourceEditor::OmSourceEditor(QWidget* parent, const char* name) : 
  QWidget(parent, name)
{

   _sourceNumber = -1;  
    


    /*****************************************************************************
     * Need the cal aperture radius (binned) for doing photometry
     ***************************************************************************/
    PhotometryParameters    photParameters = F90MODVAR(omsource_glue, myphotometryparameters);
    _calApertureRadius =  photParameters.calApertureRadius; 
   

    
 
    grpSrcProp = new QGroupBox( this, "Source number 1 properties" );
    grpSrcProp->setGeometry( QRect( 10, 20, 520, 500 ) );         // was 520 450 
    //    grpSrcProp->setGeometry( QRect( 10, 20, width() - 30, 450 ) );         // was 450 400 
  
    QGroupBox*     group = 0;
    QLabel*        label = 0;

  { // Anulli can be changed by the user
    group = new QGroupBox( grpSrcProp );
    group->setGeometry( QRect( 10, 230, 150, 120 ) ); 
    group->setTitle( tr( "Annulli" ) );

    label = new QLabel( group, "label" );
    label->setGeometry( QRect( 10, 30, 80, 20 ) ); 
    //    label->setText( tr( "Aperture" ) );

    label->setText( tr( "Aperture" ) );

    /*********************************************************
    * The spin box to change the aperture radius
    ********************************************************/

    _spinBoxApertureRadius = new QSpinBox(group, "aperture radius");


    _spinBoxApertureRadius->setGeometry(90, 30, 40, 25);
  
   
    QObject::connect(_spinBoxApertureRadius, SIGNAL(valueChanged(int)), this, SLOT(  apertureRadiusHasChanged(int)));
    QToolTip::add( _spinBoxApertureRadius, QRect( 0, 0, _spinBoxApertureRadius->width() / 2, 
						     _spinBoxApertureRadius->height() ),
						     "Aperture radius (pixels)");

    
    QLabel* label1 = new QLabel( group, "label" );
    label1->setGeometry( QRect( 10, 60, 80, 20 ) ); 
    label1->setText( tr( "BackGr r0" ) );
    //    _innerBg=new OmValEit(_info->parameter("innerbg"),group);
    //    _innerBg->setGeometry( QRect( 90, 50, 50, 25 ) ); 
    


    /*********************************************************
    * The spin box to change the inner aperture radius
    ********************************************************/

    _spinBoxInnerAnnulusRadius = new QSpinBox(group, "inner annulus radius");
    _spinBoxInnerAnnulusRadius->setGeometry(90, 60, 40, 25);

   

    QObject::connect(_spinBoxInnerAnnulusRadius,  SIGNAL(valueChanged(int)), 
           this, SLOT(  innerAnnulusRadiusHasChanged(int)));
	
    QToolTip::add( _spinBoxInnerAnnulusRadius, QRect( 0, 0, _spinBoxInnerAnnulusRadius->width() / 2, 
						     _spinBoxInnerAnnulusRadius->height() ),
						     "Background inner annulus radius (pixels)");

    QLabel* label2 = new QLabel( group, "label" );
    label2->setGeometry( QRect( 10, 90, 80, 20 ) ); 
    label2->setText( tr( "BackGr r1" ) );

    /*********************************************************
    * The spin box to change the outer aperture radius
    ********************************************************/

    _spinBoxOuterAnnulusRadius = new QSpinBox(group, "outer annulus radius");
    _spinBoxOuterAnnulusRadius ->setGeometry(90, 90, 40, 25);
    QToolTip::add( _spinBoxOuterAnnulusRadius, QRect( 0, 0, _spinBoxOuterAnnulusRadius->width() / 2, 
						     _spinBoxOuterAnnulusRadius->height() ),
						     "Background outer annulus radius (pixels)");
  
    QObject::connect(_spinBoxOuterAnnulusRadius, SIGNAL(valueChanged(int)), 
           this, SLOT(  outerAnnulusRadiusHasChanged(int)));
	
    //    initApertureSpinBoxes();

  }

  { // Pixel Coordinates
    group = new QGroupBox( grpSrcProp );
    group->setGeometry( QRect( 10, 20, 150, 90 ) ); 
    group->setTitle( tr( "Pixel Coordinates" ) );
    
    QLabel* label = new QLabel( group, "label" );
    label->setGeometry( QRect( 10, 25, 40, 20 ) ); 
    label->setText( tr( "X" ) );
    _xCoord = new QLineEdit(group, "xcoord");
    _xCoord->setReadOnly( true );
    _xCoord->setGeometry( QRect( 60, 20, 80, 25 ) ); 

    QLabel* label1 = new QLabel( group, "label" );
    label1->setGeometry( QRect( 10, 55, 40, 20 ) ); 
    label1->setText( tr( "Y" ) );

    _yCoord = new QLineEdit(group, "ycoord");
    _yCoord->setReadOnly( true );
    _yCoord->setGeometry( QRect( 60, 50, 80, 25 ) ); 
  }

  { // Sky Coordinates

    group = new QGroupBox( grpSrcProp );
    group->setGeometry( QRect( 10, 120, 150, 90 ) ); 
    group->setTitle( tr( "Sky Coordinates" ) );

    QLabel* label2 = new QLabel( group, "label" );
    label2->setGeometry( QRect( 10, 25, 40, 20 ) ); 
    label2->setText( tr( "RA" ) );
    _ascension = new QLineEdit(group, "ascension");
    _ascension->setReadOnly( true );
    _ascension->setGeometry( QRect( 55, 15, 90, 25 ) ); 

    QLabel* label3 = new QLabel( group, "label" );
    label3->setGeometry( QRect( 10, 55, 40, 20 ) ); 
    label3->setText( tr( "DEC" ) );

    _declination = new QLineEdit(group, "declination");
    _declination->setReadOnly( true );
    _declination->setGeometry( QRect( 55, 55, 90, 25 ) ); 
  }

  { // Rates
    group = new QGroupBox( grpSrcProp );
    group->setGeometry( QRect( 170, 20, 285, 120 ) );  // was 270 
    group->setTitle( tr( "Count Rates" ) );

    QLabel* label = new QLabel( group, "label" );
    label->setGeometry( QRect( 10, 20, 120, 20 ) ); 
    label->setText( tr( "Raw" ) );
    _rateRaw = new QLineEdit(group, "rateraw");
    _rateRaw->setReadOnly( true );
    _rateRaw->setGeometry( QRect( 140, 20, 60, 25 ) ); 

    QLabel* label1 = new QLabel( group, "label" );
    label1->setGeometry( QRect( 200, 20, 120, 20 ) ); 
    label1->setText( tr( "~" ) );
    _rateRawErr =  new QLineEdit(group, "raterawerr");
    _rateRawErr->setReadOnly( true );
    _rateRawErr->setGeometry( QRect( 210, 20, 60, 25 ) ); 

    QLabel* label2 = new QLabel( group, "label" );
    label2->setGeometry( QRect( 10, 50, 120, 20 ) ); 
    label2->setText( tr( "Corrected" ) );
    _rateCor = new QLineEdit(group, "ratecount");
    _rateCor->setReadOnly( true );
    _rateCor->setGeometry( QRect( 140, 50, 60, 25 ) ); 

    QLabel* label3 = new QLabel( group, "label" );
    label3->setGeometry( QRect( 200, 50, 20, 20 ) ); 
    label3->setText( tr( "~" ) );
    _rateCorErr =  new QLineEdit(group, "ratecounterr");
    _rateCorErr->setReadOnly( true );
    _rateCorErr->setGeometry( QRect( 210, 50, 60, 25 ) ); 

    QLabel* label4 = new QLabel( group, "label" );
    label4->setGeometry( QRect( 10, 80, 120, 20 ) ); 
    label4->setText( tr( "Corr. Background" ) );
    _rateBgr = new QLineEdit(group, "ratebgr");
    _rateBgr->setReadOnly( true );
    _rateBgr->setGeometry( QRect( 140, 80, 60, 25 ) ); 

   
  }

  { // Instrumental
    group = new QGroupBox( grpSrcProp, "group" );
    group->setGeometry( QRect( 170, 160, 285, 180 ) );   // was 270
    group->setTitle( tr( "Instrumental" ) );

    QLabel* label = new QLabel( group, "label" );
    label->setGeometry( QRect( 10, 20, 120, 20 ) ); 
    label->setText( tr( "Magnitude" ) );
    _magnitude = new QLineEdit(group, "magnitude");
    _magnitude->setReadOnly( true );
    _magnitude->setGeometry( QRect( 140, 20, 60, 25 ) ); 


    QLabel* label1 = new QLabel( group, "label" );
    label1->setGeometry( QRect( 200, 20, 20, 20 ) ); 
  
    label1->setText( tr( "~" ) );


    _magnitudeErr = new QLineEdit(group, "magnitudeerr");
    _magnitudeErr->setReadOnly( true );
    _magnitudeErr->setGeometry( QRect( 210, 20, 60, 25 ) ); 

    QLabel* label2 = new QLabel( group, "label" );
    label2->setGeometry( QRect( 10, 50, 120, 20 ) ); 
    label2->setText( tr( "Significance" ) );
    _significance = new QLineEdit(group, "significance");
    _significance->setReadOnly( true );
    _significance->setGeometry( QRect( 140, 50, 60, 25 ) ); 

    QLabel* label3 = new QLabel( group, "label" );
    label3->setGeometry( QRect( 10, 80, 120, 20 ) ); 
    //    label->setText( tr( "Coincidence loss" ) );
    label3->setText( tr( "Coincidence loss corr" ) );

    _coincidenceLoss = new QLineEdit(group, "coincidenceloss");
    _coincidenceLoss->setReadOnly( true );
    _coincidenceLoss->setGeometry( QRect( 140, 80, 60, 25 ) ); 

    QLabel* label4 = new QLabel( group, "label" );
    label4->setGeometry( QRect( 10, 110, 120, 20 ) ); 
  
    label4->setText( tr( "Psf1 corr" ) );


    _psf1 = new QLineEdit(group, "psf1");
    _psf1->setReadOnly( true );
    _psf1->setGeometry( QRect( 140, 110, 60, 25 ) ); 

    QLabel* label5 = new QLabel( group, "label" );
    label5->setGeometry( QRect( 10, 140, 120, 20 ) ); 
   
    label5->setText( tr( "Psf2 corr" ) );

    _psf2 = new QLineEdit(group, "psf2");
    _psf2->setReadOnly( true );
    _psf2->setGeometry( QRect( 140, 140, 60, 25 ) ); 
  }

  { // the buttons

    _applyBtn = new QPushButton( grpSrcProp, "_applyBtn" );
    _applyBtn->setEnabled(false);
    _applyBtn->setGeometry( QRect( 280, 350, 70, 35 ) ); 
    _applyBtn->setText( tr( "Apply" ) );

    QToolTip::add( _applyBtn , QRect( 0, 0,   _applyBtn->width() / 2, 
	 _applyBtn->height() ), "Recompute the instrumental magnitude using the selected aperture and background annulus");



    
    _deleteBtn = new QPushButton( grpSrcProp, "_deleteBtn" );
    _deleteBtn->setEnabled( false );
    _deleteBtn->setGeometry( QRect( 200, 350, 70, 35 ) ); 
    _deleteBtn->setText( tr( "Delete" ) );
    QToolTip::add( _deleteBtn , QRect( 0, 0,   _deleteBtn->width() / 2, 
	 _deleteBtn->height() ), "Delete the selected source");

    


    _cancelBtn = new QPushButton( grpSrcProp, "_cancelBtn" );
    _cancelBtn->setEnabled(false);
    _cancelBtn->setGeometry( QRect( 360, 350, 80, 35 ) ); 
    _cancelBtn->setText( tr( "Cancel" ) );

    connect(_applyBtn,SIGNAL(clicked()),this,SLOT(applyClicked()));
    connect(_deleteBtn,SIGNAL(clicked()),this,SLOT(deleteClicked()));
    connect(_cancelBtn,SIGNAL(clicked()),this,SLOT(cancelClicked()));
  }
 
  //resize(sizeHint());
    setEnabled(false);
    
}
/****************************************************
*
****************************************************/

OmSourceEditor::~OmSourceEditor()
{
  
}
/****************************************************
*
****************************************************/
void OmSourceEditor::sourceHasChanged(bool changed)
{
 
  _cancelBtn->setEnabled(changed);
  _applyBtn->setEnabled(changed);
  _deleteBtn->setEnabled(!changed);  
}
/******************************************************
* initApertureSpinBoxes
* Purpose- Initialises the spin boxes with the default
* values
******************************************************/

void OmSourceEditor::initApertureSpinBoxes()
{

 
    _spinBoxApertureRadius->setMaxValue((int)_calApertureRadius);
 

    _spinBoxApertureRadius->setMinValue((int)(_calApertureRadius / 2.0));

    _spinBoxApertureRadius->setValue((int)(_calApertureRadius));
  

    
    _spinBoxInnerAnnulusRadius->setMaxValue((int)_calApertureRadius + 22);
  

    _spinBoxInnerAnnulusRadius->setMinValue((int)_calApertureRadius + 2);
  

    _spinBoxInnerAnnulusRadius->setValue((int)_calApertureRadius + 2);
 
  
    _spinBoxOuterAnnulusRadius->setMaxValue((int)_calApertureRadius + 22);
 

    _spinBoxOuterAnnulusRadius->setMinValue((int)_calApertureRadius  + 6);
  

    _spinBoxOuterAnnulusRadius->setValue((int)_calApertureRadius + 6);

}
/******************************************************
* sourceHasChanged
* Purpose- called when user activates a spin button control
******************************************************/
void OmSourceEditor::apertureRadiusHasChanged(int value)
{
 
    _cancelBtn->setEnabled(true);
    _applyBtn->setEnabled(true);
    _deleteBtn->setEnabled(false);  

  
    _current.photometryradius = value;
 
}
/******************************************************
* innerAnnulusRadiusGasChanged
* Purpose- called when user activates a spin button control
******************************************************/
void OmSourceEditor::innerAnnulusRadiusHasChanged(int value)
{
  
    _cancelBtn->setEnabled(true);
    _applyBtn->setEnabled(true);
    _deleteBtn->setEnabled(false);
 

    _spinBoxOuterAnnulusRadius->setMinValue(value + 4);
    _spinBoxOuterAnnulusRadius->setMaxValue(value + 20);
    _spinBoxOuterAnnulusRadius->setValue(value + 4);


    _current.innerbackgroundradius = value;
    _current.outerbackgroundradius = value + 4;
  
}
/******************************************************
* outerAnnulusRadiusGasChanged
* Purpose- called when user activates a spin button control
******************************************************/
void OmSourceEditor::outerAnnulusRadiusHasChanged(int value)
{
  
    _cancelBtn->setEnabled(true);
    _applyBtn->setEnabled(true);
    _deleteBtn->setEnabled(false);
  
    _current.outerbackgroundradius = value;
   
}
/******************************************************************
* edit
*****************************************************************/
void OmSourceEditor::edit(const int number, const OmSourceParameter& s)
{
     if (_sourceNumber != number)
         initApertureSpinBoxes(); 

    _sourceNumber = number;
    _current = s;


  

   
    QString string;
   
    string.sprintf("Source editor- editing source no %d", _sourceNumber);
  
    grpSrcProp->setTitle(string);


    source2editor();

    sourceHasChanged(false);
    setEnabled(true);
  

}
/***********************************************
* source2editor
***********************************************/
void OmSourceEditor::source2editor()
{
 
    QString text;
    _xCoord->setText(text.sprintf("%9.1f",  _current.xcoord));

    _yCoord->setText(text.sprintf("%9.1f",  _current.ycoord));
  
    AngleFormatter af;
    af.setMode(AngleFormatter::HMS);
    _ascension->setText(af.asString(_current.rightascension).c_str());
    af.setMode(AngleFormatter::DMS);
    _declination->setText(af.asString(_current.declination).c_str());

    _rateCor->setText(text.sprintf("%8.2f",  _current.countrate));
  
    _rateCorErr->setText(text.sprintf("%8.2f",  _current.countrateerror));

    _rateRaw->setText(text.sprintf("%8.2f",  _current.rawcountrate));

    _rateRawErr->setText(text.sprintf("%8.2f",  _current.rawcountrateerror));

    _rateBgr->setText(text.sprintf("%8.2f",  _current.backgroundcountrate));

    _magnitude->setText(text.sprintf("%8.2f",  _current.magnitude));


    _magnitudeErr->setText(text.sprintf("%8.2f",  _current.magnitudeerror));
 
    _significance->setText(text.sprintf("%8.2f",  _current.significance));


    _coincidenceLoss->setText(text.sprintf("%8.2f",  _current.coincidencecorrection));

 

  _psf1->setText(text.sprintf("%8.2f",  _current.psf1correction));

 

 //  pretty(_psfY,_current.psf2correction);


  _psf2->setText(text.sprintf("%8.2f",  _current.psf2correction));
  //  _aperture->parameter()->addView(this);
  //  _innerBg->parameter()->addView(this);
  //  _outerBg->parameter()->addView(this);
}
/****************************************************
*
*
****************************************************/
void OmSourceEditor::editor2source()
{


}
/****************************************************
*
*
****************************************************/
void OmSourceEditor::applyClicked()
{
  editor2source();
  emit changed(_current);
}

void OmSourceEditor::cancelClicked()
{
  source2editor();
  sourceHasChanged(false);
}
/****************************************************
*
*
****************************************************/

void OmSourceEditor::deleteClicked()
{
  emit requestedDelete();
}

/****************************************************
*
*
****************************************************/
void OmSourceEditor::updateView()
{
  sourceHasChanged(true);  

}











