// -*-C++-*-
//
// File:      OmSourceEditor.h
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Thu Sep  5 14:06:31 MEST 2002
// Usage:     -
// Info:      OmSourceEditor is presents the properties of an OmSource in a
//            widget for editing. It allows to change the anulli, that will
//            be use to recalculate the psf of the source and its magnitude.
//            
#ifndef OmSourceEditor_h
#define OmSourceEditor_h
#include "OmSourceParameters.h"
#include "ParameterView.h"
#include <qwidget.h>


class QLineEdit;;
class QPushButton;
class QGroupBox;
class QSpinBox;

class OmSourceEditor : public QWidget, public ParameterView
{
  Q_OBJECT
public:
  OmSourceEditor(QWidget* parent=0, const char* name=0);
  virtual ~OmSourceEditor();

signals:
  // broadcast if contents changed
  void changed(const OmSourceParameter&);

  //
  void requestedDelete();

public  slots:
  void edit(const int number, const OmSourceParameter&);

private slots:
  void applyClicked();
  void cancelClicked();
  void deleteClicked();
  void apertureRadiusHasChanged(int value);        // Spin box to set aperture radius
  void innerAnnulusRadiusHasChanged(int value);    // Spin box for background inner annulus radius
  void outerAnnulusRadiusHasChanged(int value);    // Spin box for background outer annulus radius



protected:
  void updateView(); // implementation of ParameterView!

private:

  


  // member functions that are disabled
  OmSourceEditor(const OmSourceEditor&);
  OmSourceEditor& operator=(const OmSourceEditor&);
 
  // member functions
  void source2editor();
  void editor2source();
  void initialiseParameters();
  void sourceHasChanged(bool changed);
  void initApertureSpinBoxes();


  // member data
  float _calApertureRadius; 
  int _sourceNumber;
  OmSourceParameter _current;
 
  QGroupBox* grpSrcProp;

  QSpinBox* _spinBoxApertureRadius;
  QSpinBox* _spinBoxInnerAnnulusRadius;
  QSpinBox* _spinBoxOuterAnnulusRadius;



  QLineEdit*  _xCoord;
  QLineEdit*  _yCoord;
  QLineEdit*  _ascension;
  QLineEdit*  _declination;

  QLineEdit*  _rateCor;
  QLineEdit*  _rateCorErr;
  QLineEdit*  _rateBgr;
  QLineEdit*  _rateBgrErr;
  QLineEdit*  _rateRaw;
  QLineEdit*  _rateRawErr;

  QLineEdit*  _magnitude;
  QLineEdit*  _magnitudeErr;
  QLineEdit*  _significance;
  QLineEdit*  _coincidenceLoss;
  QLineEdit*  _psf1;
  QLineEdit*  _psf2;

  QPushButton* _applyBtn;
  QPushButton* _deleteBtn;
  QPushButton* _cancelBtn;
};



#endif
