! -*-f90-*-
!
! Author:    jbakker
! Generated: Wed Aug  9 17:30:51 MEST 2002
! Info:      interface for GUI c++code
! Note:
!  * The GUI should always call "init()" first, and "finish()" last.
!  * In comments, I usually talk about methods. A method may be either a
!    function OR subroutine within a module
!  * Any variable starting with 'my' is global to this module
!  * Please read the comments below ;-)

module omsource_glue
   use param

   use omsource_old
   use omsourceparameter_mod
   use omsourceregion_mod
   use omprogress_mod
   implicit none
   
   type(OmSourceParameter), pointer :: mySources(:)

   character(256) :: myImageFileName
   character(256) :: myOldSourceListFile = " "

   character(256) :: myNewSourceListFile

   real ::  mycalapertureradius
   type(ObsDetails)  :: myObsDetails
   type(ImageWindow) :: myImageWindow
   type(PhotometryParameters) :: myPhotometryParameters
contains

   ! Info: This is always the first function that is called by the GUI;
   !       in here we initialise/load/whatever




   subroutine init(fileName)
    
      character(256), intent(in) :: fileName  
       
      myOldSourceListFile = stringParameter("oldsrclistset")
      myNewSourceListFile = stringParameter("newsrclistset")



      call omsource_init(mySources, myPhotometryParameters, &
                         myObsDetails, myImageWindow,      &
                         fileName, myOldSourceListFile)
   end subroutine init

   subroutine setImageFileName(fileName)
       character(256), intent(in) :: fileName
       
       myImageFileName = fileName
       
   end subroutine setImageFileName

    subroutine setNewSourceListFileName(fileName)
       character(256), intent(in) :: fileName
 

       myNewSourceListFile = fileName

    end subroutine setNewSourceListFileName
    subroutine setOldSourceListFileName(fileName)
       character(256), intent(in) :: fileName
 

!       myOldSourceListFileName = fileName


    end subroutine setOldSourceListFileName



!  subroutine finish()
     
!      call omsource_finish(mySources)
 
!   end subroutine finish


   ! Info: If the GUI closes, it calls
   subroutine finish(sourceListFile)
      character(*), intent(in) :: sourceListFile

      call omsource_finish(mySources,  myNewSourceListFile)
 
   end subroutine finish



   ! Subroutine for loading the sources from the source-list file
   ! produced by omdetect in omsourcebrower::SourceDetection
   subroutine loadsources(sourceListFile)
      character(*), intent(in) :: sourceListFile
      call omsource_loadsources(mySources, sourceListFile)
   end subroutine loadsources





   ! Info: Recompute a single source as aperture, and background radii
   !       may have changed
   subroutine recompute(id)
      integer, intent(in) :: id
      
      call omsource_recompute(mySources, id)
   end subroutine recompute



   ! Info: Remove a single source at location id
   subroutine remove(id)
      integer, intent(in) :: id
      
      call omsource_delete(mySources, id)
   end subroutine remove



   ! Info: User has selected regions in the image; now it is up to us to:
   !       - identify regions that are not found in the sourcelist already
   !       - enlarge the sourcelist with information within the regions
   !       - compute the source properties for the added sources
   subroutine addRegions(regions)
      type(OmSourceRegion),intent(in) :: regions(:)

      call omsource_add(mySources, regions)
   end subroutine addRegions
   
end module omsource_glue








